import {legend} from "./country-legend.js";
import {width, height, margin, colors} from "./constants.js";


function resize() {
    const container = document.getElementById('time-series-container');
    const width = container.clientWidth - margin.left - margin.right;
    const height = container.clientHeight - margin.top - margin.bottom;
    return { width, height };
}

d3.csv("schengen_data.csv").then((data) => {
    const { width, height } = resize();
    const allStates = new Set(data.map(d => d["Schengen State"]));
    const color = d3
        .scaleOrdinal()
        .domain(data)
        .range(colors)

    const svg = d3.select("#time-series-container")
        .append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom)
        .append("g")
        .attr("transform", `translate(${margin.left+margin.right},${margin.top+margin.bottom})`);

    function updateGraph(type) {
        const processedData = d3.rollups(
            data,
            v => d3.sum(v, d => +d[type] || 0),
            d => +d["Year"],
            d => d["Schengen State"]
        ).sort((a, b) => a[0] - b[0]); // Sort by year

        const aggregatedData = processedData.map(([year, states]) => {
            const yearObj = { year: +year };
            for (const [state, sum] of states) {
                yearObj[state] = sum;
            }
            return yearObj;
        });

        const x = d3.scaleLinear()
            .domain(d3.extent(aggregatedData, d => d.year))
            .range([0, width-margin.left-margin.right]);

        const y = d3.scaleLinear()
            .domain([0, d3.max(aggregatedData, d =>
                d3.max([...allStates].map(state => d[state] || 0))
            )])
            .nice()
            .range([height-margin.top-margin.bottom, 0]);

        svg.selectAll(".axis").remove();
        svg.selectAll(".line").remove();

        svg.append("g")
            .attr("transform", `translate(0,${height-margin.top-margin.bottom})`)
            .attr("class", "axis")
            .call(d3.axisBottom(x).tickFormat(d3.format("d")));

        svg.append("g")
            .attr("class", "axis")
            .call(d3.axisLeft(y));

        for (const state of allStates) {
            const stateData = aggregatedData.map(d => ({ year: d.year, value: d[state] || 0 }));

            svg.append("path")
                .datum(stateData)
                .attr("class", "line")
                .attr("d", d3.line()
                    .x(d => x(d.year))
                    .y(d => y(d.value))
                )
                .attr("stroke", color(state))
                .attr("stroke-width", 1.5)
                .attr("fill", "none")
                .on("mouseover", function() {
                    d3.select(this).classed("highlighted", true);
                })
                .on("mouseout", function() {
                    d3.select(this).classed("highlighted", false);
                })
        }

        legend(svg, allStates, color)
    }

    document.getElementById("visa-type-selector").addEventListener("change", (event) => {
        const type = event.target.value;
        updateGraph(type);
    });

    // Initialize with "Total  uniform visas issued (including MEV)"
    updateGraph("Total  uniform visas issued (including MEV)");
});